# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
#
# Copyright 2017-2023 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""OS release information helpers."""

import contextlib
from pathlib import Path
from typing import Dict

from craft_archives import errors

_ID_TO_UBUNTU_CODENAME = {
    "17.10": "artful",
    "17.04": "zesty",
    "16.04": "xenial",
    "14.04": "trusty",
}


class OsRelease:
    """A class to intelligently determine the OS on which we're running."""

    def __init__(
        self, *, os_release_file: Path = Path("/etc/os-release")  # noqa: B008
    ) -> None:
        """Create a new OsRelease instance.

        :param str os_release_file: Path to os-release file to be parsed.
        """
        with contextlib.suppress(FileNotFoundError):
            self._os_release: Dict[str, str] = {}
            with os_release_file.open(encoding="utf-8") as release_file:
                for line in release_file:
                    entry = line.rstrip().split("=")
                    if len(entry) == 2:  # noqa: PLR2004
                        self._os_release[entry[0]] = entry[1].strip('"')

    def id(self) -> str:
        """Return the OS ID.

        :raises SnapcraftError: If no ID can be determined.
        """
        with contextlib.suppress(KeyError):
            return self._os_release["ID"]

        raise errors.ArchivesError("Unable to determine host OS ID")

    def name(self) -> str:
        """Return the OS name.

        :raises SnapcraftError: If no name can be determined.
        """
        with contextlib.suppress(KeyError):
            return self._os_release["NAME"]

        raise errors.ArchivesError("Unable to determine host OS name")

    def version_id(self) -> str:
        """Return the OS version ID.

        :raises SnapcraftError: If no version ID can be determined.
        """
        with contextlib.suppress(KeyError):
            return self._os_release["VERSION_ID"]

        raise errors.ArchivesError("Unable to determine host OS version ID")

    def version_codename(self) -> str:
        """Return the OS version codename.

        This first tries to use the VERSION_CODENAME. If that's missing, it
        tries to use the VERSION_ID to figure out the codename on its own.

        :raises SnapcraftError: If no version codename can be determined.
        """
        with contextlib.suppress(KeyError):
            return self._os_release["VERSION_CODENAME"]

        with contextlib.suppress(KeyError):
            return _ID_TO_UBUNTU_CODENAME[self._os_release["VERSION_ID"]]

        raise errors.ArchivesError("Unable to determine host OS version codename")



# This file is part of craft-archives.
#
# Copyright 2023 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""craft-archives package demo."""
from typing import List, Optional, Any


def hello(people: Optional[List[Any]] = None) -> None:
    """Says hello."""
    print("Hello *craft team!")
    if people:
        for person in people:
            print(f"Hello {person}!")



# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
#
# Copyright 2019-2023 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Project model definitions and helpers."""

from typing import TYPE_CHECKING, Any, Dict, List, Literal, Optional

import pydantic
from pydantic import constr

# Workaround for mypy
# see https://github.com/samuelcolvin/pydantic/issues/975#issuecomment-551147305
if TYPE_CHECKING:
    KeyIdStr = str
else:
    KeyIdStr = constr(regex=r"^[0-9A-F]{40}$")


class ProjectModel(pydantic.BaseModel):
    """Base model for project repository classes."""

    class Config:  # pylint: disable=too-few-public-methods
        """Pydantic model configuration."""

        # pyright: reportUnknownMemberType=false
        # pyright: reportUnknownVariableType=false
        # pyright: reportUnknownLambdaType=false

        validate_assignment = True
        allow_mutation = False
        allow_population_by_field_name = True
        alias_generator = lambda s: s.replace("_", "-")  # noqa: E731
        extra = "forbid"


# TODO: Project repo definitions are almost the same as PackageRepository
#       ported from legacy. Check if we can consolidate them and remove
#       field validation (moving all validation rules to pydantic).


class AptDeb(ProjectModel):
    """Apt package repository definition."""

    type: Literal["apt"]
    url: str
    key_id: KeyIdStr
    architectures: Optional[List[str]]
    formats: Optional[List[Literal["deb", "deb-src"]]]
    components: Optional[List[str]]
    key_server: Optional[str]
    path: Optional[str]
    suites: Optional[List[str]]

    @classmethod
    def unmarshal(cls, data: Dict[str, Any]) -> "AptDeb":
        """Create an AptDeb object from dictionary data."""
        return cls(**data)


class AptPPA(ProjectModel):
    """PPA package repository definition."""

    type: Literal["apt"]
    ppa: str

    @classmethod
    def unmarshal(cls, data: Dict[str, Any]) -> "AptPPA":
        """Create an AptPPA object from dictionary data."""
        return cls(**data)


def validate_repository(data: Dict[str, Any]) -> None:
    """Validate a package repository.

    :param data: The repository data to validate.
    """
    if not isinstance(data, dict):  # pyright: reportUnnecessaryIsInstance=false
        raise TypeError("value must be a dictionary")

    try:
        AptPPA(**data)
        return
    except pydantic.ValidationError:
        pass

    AptDeb(**data)
